#!/usr/sbin/dtrace -s

/*
 * timestamp-sanity-checks.d
 *
 * Sanity-check timestamp and walltimestamp deltas. For details see:
 * http://www.fabiankeil.de/gehacktes/dtrace-timestamp-tests/
 *
 * Copyright (c) 2012 Fabian Keil <fk@fabiankeil.de>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#pragma D option quiet

dtrace:::BEGIN
{
        last_walltimestamp = 0;
        last_timestamp = 0;
}

syscall:::entry
{
        @w["total"] = count();
        this->walltimestamp = walltimestamp;
        this->last_walltimestamp = last_walltimestamp;

        @t["total"] = count();
        this->timestamp = timestamp;
        this->last_timestamp = last_timestamp;
}

syscall:::entry
/this->last_walltimestamp > this->walltimestamp/
{
        @w["backwards"] = count();
}

syscall:::entry
/this->last_walltimestamp < this->walltimestamp/
{
        @w["forward"] = count();
}

syscall:::entry
/this->last_walltimestamp == this->walltimestamp/
{
        @w["no change"] = count();
}

syscall:::entry
/this->last_timestamp > this->timestamp/
{
        @t["backwards"] = count();
}

syscall:::entry
/this->last_timestamp < this->timestamp/
{
        @t["forward"] = count();
}

syscall:::entry
/this->last_timestamp == this->timestamp/
{
        @t["no change"] = count();
}

syscall:::entry
{
        last_walltimestamp = this->walltimestamp;
        last_timestamp = this->timestamp;
}

tick-60sec,
dtrace:::END
{
        printf("-------------- %Y --------------\n", walltimestamp);
        printf("walltimestamp:\n");
        printa(@w);
        printf("\ntimestamp:\n");
        printa(@t);
}
